@@ -61,7 +61,9 @@ if (empty($time)) {
     $time = time();
 }
 
-if ($courseid != SITEID) {
+$iscoursecalendar = $courseid != SITEID;
+
+if ($iscoursecalendar) {
     $url->param('course', $courseid);
 }
 
@@ -80,7 +82,7 @@ $PAGE->set_url($url);
 
 $course = get_course($courseid);
 
-if ($courseid != SITEID && !empty($courseid)) {
+if ($iscoursecalendar && !empty($courseid)) {
     navigation_node::override_active_url(new moodle_url('/course/view.php', array('id' => $course->id)));
 } else if (!empty($categoryid)) {
     core_course_category::get($categoryid); // Check that category exists and can be accessed.
@@ -115,7 +117,10 @@ switch($view) {
 // Print title and header
 $PAGE->set_pagelayout('standard');
 $PAGE->set_title("$course->shortname: $strcalendar: $pagetitle");
-$PAGE->set_heading($COURSE->fullname);
+
+$headingstr = ($iscoursecalendar) ? get_string('coursecalendar', 'core_calendar', $COURSE->shortname) :
+        get_string('calendar', 'core_calendar');
+$PAGE->set_heading($headingstr);
 
 $renderer = $PAGE->get_renderer('core_calendar');
 $calendar->add_sidecalendar_blocks($renderer, true, $view);
@@ -123,7 +128,7 @@ $calendar->add_sidecalendar_blocks($renderer, true, $view);
 echo $OUTPUT->header();
 echo $renderer->start_layout();
 echo html_writer::start_tag('div', array('class'=>'heightcontainer'));
-echo $OUTPUT->heading(get_string('calendar', 'calendar'));
+
 
 
 list($data, $template) = calendar_get_view($calendar, $view, true, false, $lookahead);
