@@ -172,6 +172,42 @@ class question_usage_by_activity {
         return $qa->get_slot();
     }
 
+    /**
+     * Add another question to this usage, in the place of an existing slot.
+     * The question_attempt that was in that slot is moved to the end at a new
+     * slot number, which is returned.
+     *
+     * The added question is not started until you call {@link start_question()}
+     * on it.
+     *
+     * @param int $slot the slot-number of the question to replace.
+     * @param question_definition $question the question to add.
+     * @param number $maxmark the maximum this question will be marked out of in
+     *      this attempt (optional). If not given, the max mark from the $qa we
+     *      are replacing is used.
+     * @return int the new slot number of the question that was displaced.
+     */
+    public function add_question_in_place_of_other($slot, question_definition $question, $maxmark = null) {
+        $newslot = $this->next_slot_number();
+
+        $oldqa = $this->get_question_attempt($slot);
+        $oldqa->set_slot($newslot);
+        $this->questionattempts[$newslot] = $oldqa;
+
+        if ($maxmark === null) {
+            $maxmark = $oldqa->get_max_mark();
+        }
+
+        $qa = new question_attempt($question, $this->get_id(), $this->observer, $maxmark);
+        $qa->set_slot($slot);
+        $this->questionattempts[$slot] = $qa;
+
+        $this->observer->notify_attempt_moved($oldqa, $slot);
+        $this->observer->notify_attempt_added($qa);
+
+        return $newslot;
+    }
+
     /**
      * The slot number that will be allotted to the next question added.
      */
@@ -271,6 +307,17 @@ class question_usage_by_activity {
         return $this->get_question_attempt($slot)->get_state_class($showcorrectness);
     }
 
+    /**
+     * Whether this attempt at a given question could be completed just by the
+     * student interacting with the question, before {@link finish_question()} is called.
+     *
+     * @param int $slot the number used to identify this question within this usage.
+     * @return boolean whether the attempt at the given question can finish naturally.
+     */
+    public function can_question_finish_during_attempt($slot) {
+        return $this->get_question_attempt($slot)->can_finish_during_attempt();
+    }
+
     /**
      * Get the time of the most recent action performed on a question.
      * @param int $slot the number used to identify this question within this usage.
@@ -310,10 +357,8 @@ class question_usage_by_activity {
     }
 
     /**
-     * Get the current mark awarded for the attempt at a question.
-     * @param int $slot the number used to identify this question within this usage.
-     * @return number|null The current mark for this question, or null if one has
-     * not been assigned yet.
+     * Get the total mark for all questions in this usage.
+     * @return number The sum of marks of all the question_attempts in this usage.
      */
     public function get_total_mark() {
         $mark = 0;
@@ -340,6 +385,7 @@ class question_usage_by_activity {
      * The values are arrays with two items, title and content. Each of these
      * will be either a string, or a renderable.
      *
+     * @param question_display_options $options display options to apply.
      * @return array as described above.
      */
     public function get_summary_information(question_display_options $options) {
@@ -348,26 +394,56 @@ class question_usage_by_activity {
     }
 
     /**
-     * @return string a simple textual summary of the question that was asked.
+     * Get a simple textual summary of the question that was asked.
+     *
+     * @param int $slot the slot number of the question to summarise.
+     * @return string the question summary.
      */
     public function get_question_summary($slot) {
         return $this->get_question_attempt($slot)->get_question_summary();
     }
 
     /**
-     * @return string a simple textual summary of response given.
+     * Get a simple textual summary of response given.
+     *
+     * @param int $slot the slot number of the question to get the response summary for.
+     * @return string the response summary.
      */
     public function get_response_summary($slot) {
         return $this->get_question_attempt($slot)->get_response_summary();
     }
 
     /**
-     * @return string a simple textual summary of the correct resonse.
+     * Get a simple textual summary of the correct response to a question.
+     *
+     * @param int $slot the slot number of the question to get the right answer summary for.
+     * @return string the right answer summary.
      */
     public function get_right_answer_summary($slot) {
         return $this->get_question_attempt($slot)->get_right_answer_summary();
     }
 
+    /**
+     * Return one of the bits of metadata for a particular question attempt in
+     * this usage.
+     * @param int $slot the slot number of the question of inereest.
+     * @param string $name the name of the metadata variable to return.
+     * @return string the value of that metadata variable.
+     */
+    public function get_question_attempt_metadata($slot, $name) {
+        return $this->get_question_attempt($slot)->get_metadata($name);
+    }
+
+    /**
+     * Set some metadata for a particular question attempt in this usage.
+     * @param int $slot the slot number of the question of inerest.
+     * @param string $name the name of the metadata variable to return.
+     * @param string $value the value to set that metadata variable to.
+     */
+    public function set_question_attempt_metadata($slot, $name, $value) {
+        $this->get_question_attempt($slot)->set_metadata($name, $value);
+    }
+
     /**
      * Get the {@link core_question_renderer}, in collaboration with appropriate
      * {@link qbehaviour_renderer} and {@link qtype_renderer} subclasses, to generate the
@@ -433,9 +509,8 @@ class question_usage_by_activity {
      * For internal use only. Used when reloading the state of a question from the
      * database.
      *
-     * @param array $records Raw records loaded from the database.
-     * @param int $questionattemptid The id of the question_attempt to extract.
-     * @return question_attempt The newly constructed question_attempt_step.
+     * @param int $slot the slot number of the question to replace.
+     * @param question_attempt $qa the question attempt to put in that place.
      */
     public function replace_loaded_question_attempt_info($slot, $qa) {
         $this->check_slot($slot);
@@ -477,14 +552,15 @@ class question_usage_by_activity {
      * @param int $variant which variant of the question to use. Must be between
      *      1 and ->get_num_variants($slot) inclusive. If not give, a variant is
      *      chosen at random.
+     * @param int|null $timenow optional, the timstamp to record for this action. Defaults to now.
      */
-    public function start_question($slot, $variant = null) {
+    public function start_question($slot, $variant = null, $timenow = null) {
         if (is_null($variant)) {
             $variant = rand(1, $this->get_num_variants($slot));
         }
 
         $qa = $this->get_question_attempt($slot);
-        $qa->start($this->preferredbehaviour, $variant);
+        $qa->start($this->preferredbehaviour, $variant, array(), $timenow);
         $this->observer->notify_attempt_modified($qa);
     }
 
@@ -501,7 +577,8 @@ class question_usage_by_activity {
         }
 
         foreach ($this->questionattempts as $qa) {
-            $qa->start($this->preferredbehaviour, $qa->select_variant($variantstrategy));
+            $qa->start($this->preferredbehaviour, $qa->select_variant($variantstrategy), array(),
+                    $timestamp, $userid);
             $this->observer->notify_attempt_modified($qa);
         }
     }
@@ -599,7 +676,7 @@ class question_usage_by_activity {
      * particular question.
      *
      * @param int $slot the number used to identify this question within this usage.
-     * @param $postdata optional, only intended for testing. Use this data
+     * @param array|null $postdata optional, only intended for testing. Use this data
      * instead of the data from $_POST.
      * @return array submitted data specific to this question.
      */
@@ -623,16 +700,24 @@ class question_usage_by_activity {
 
             // Behaviour vars should not be processed by question type, just add prefix.
             $behaviourvars = $this->get_question_attempt($slot)->get_behaviour()->get_expected_data();
-            foreach ($behaviourvars as $behaviourvarname => $unused) {
-                $behaviourvarkey = '-'.$behaviourvarname;
-                if (isset($responsedata[$behaviourvarkey])) {
-                    $slotresponse[$behaviourvarkey] = $responsedata[$behaviourvarkey];
-                    unset($responsedata[$behaviourvarkey]);
+            foreach (array_keys($responsedata) as $responsedatakey) {
+                if ($responsedatakey[0] === '-') {
+                    $behaviourvarname = substr($responsedatakey, 1);
+                    if (isset($behaviourvars[$behaviourvarname])) {
+                        // Expected behaviour var found.
+                        if ($responsedata[$responsedatakey]) {
+                            // Only set the behaviour var if the column value from the cvs file is non zero.
+                            // The behaviours only look at whether the var is set or not they don't look at the value.
+                            $slotresponse[$responsedatakey] = $responsedata[$responsedatakey];
+                        }
+                    }
+                    // Remove both expected and unexpected vars from data passed to question type.
+                    unset($responsedata[$responsedatakey]);
                 }
             }
 
             $slotresponse += $this->get_question($slot)->prepare_simulated_post_data($responsedata);
-            $slotresponse[':sequencecheck'] =  $this->get_question_attempt($slot)->get_sequence_check_count();
+            $slotresponse[':sequencecheck'] = $this->get_question_attempt($slot)->get_sequence_check_count();
 
             // Add this slot's prefix to slot data.
             $prefix = $this->get_field_prefix($slot);
@@ -646,7 +731,8 @@ class question_usage_by_activity {
     /**
      * Process a specific action on a specific question.
      * @param int $slot the number used to identify this question within this usage.
-     * @param $submitteddata the submitted data that constitutes the action.
+     * @param array $submitteddata the submitted data that constitutes the action.
+     * @param int|null $timestamp (optional) the timestamp to consider 'now'.
      */
     public function process_action($slot, $submitteddata, $timestamp = null) {
         $qa = $this->get_question_attempt($slot);
@@ -657,7 +743,8 @@ class question_usage_by_activity {
     /**
      * Process an autosave action on a specific question.
      * @param int $slot the number used to identify this question within this usage.
-     * @param $submitteddata the submitted data that constitutes the action.
+     * @param array $submitteddata the submitted data that constitutes the action.
+     * @param int|null $timestamp (optional) the timestamp to consider 'now'.
      */
     public function process_autosave($slot, $submitteddata, $timestamp = null) {
         $qa = $this->get_question_attempt($slot);
@@ -667,12 +754,14 @@ class question_usage_by_activity {
     }
 
     /**
-     * Check that the sequence number, that detects weird things like the student
-     * clicking back, is OK. If the sequence check variable is not present, returns
+     * Check that the sequence number, that detects weird things like the student clicking back, is OK.
+     *
+     * If the sequence check variable is not present, returns
      * false. If the check variable is present and correct, returns true. If the
      * variable is present and wrong, throws an exception.
+     *
      * @param int $slot the number used to identify this question within this usage.
-     * @param array $submitteddata the submitted data that constitutes the action.
+     * @param array|null $postdata (optional) data to use in place of $_POST.
      * @return bool true if the check variable is present and correct. False if it
      * is missing. (Throws an exception if the check fails.)
      */
@@ -691,8 +780,9 @@ class question_usage_by_activity {
 
     /**
      * Check, based on the sequence number, whether this auto-save is still required.
+     *
      * @param int $slot the number used to identify this question within this usage.
-     * @param array $submitteddata the submitted data that constitutes the action.
+     * @param array|null $postdata the submitted data that constitutes the action.
      * @return bool true if the check variable is present and correct, otherwise false.
      */
     public function is_autosave_required($slot, $postdata = null) {
@@ -712,7 +802,7 @@ class question_usage_by_activity {
      * Update the flagged state for all question_attempts in this usage, if their
      * flagged state was changed in the request.
      *
-     * @param $postdata optional, only intended for testing. Use this data
+     * @param array|null $postdata optional, only intended for testing. Use this data
      * instead of the data from $_POST.
      */
     public function update_question_flags($postdata = null) {
@@ -748,6 +838,7 @@ class question_usage_by_activity {
      * manual grading, or changing the flag state.
      *
      * @param int $slot the number used to identify this question within this usage.
+     * @param int|null $timestamp (optional) the timestamp to consider 'now'.
      */
     public function finish_question($slot, $timestamp = null) {
         $qa = $this->get_question_attempt($slot);
@@ -758,6 +849,8 @@ class question_usage_by_activity {
     /**
      * Finish the active phase of an attempt at a question. See {@link finish_question()}
      * for a fuller description of what 'finish' means.
+     *
+     * @param int|null $timestamp (optional) the timestamp to consider 'now'.
      */
     public function finish_all_questions($timestamp = null) {
         foreach ($this->questionattempts as $qa) {
@@ -815,13 +908,22 @@ class question_usage_by_activity {
         }
     }
 
+    /**
+     * Change the max mark for this question_attempt.
+     * @param int $slot the slot number of the question of inerest.
+     * @param float $maxmark the new max mark.
+     */
+    public function set_max_mark($slot, $maxmark) {
+        $this->get_question_attempt($slot)->set_max_mark($maxmark);
+    }
+
     /**
      * Create a question_usage_by_activity from records loaded from the database.
      *
      * For internal use only.
      *
      * @param Iterator $records Raw records loaded from the database.
-     * @param int $questionattemptid The id of the question_attempt to extract.
+     * @param int $qubaid The id of the question usage we are loading.
      * @return question_usage_by_activity The newly constructed usage.
      */
     public static function load_from_records($records, $qubaid) {
@@ -829,7 +931,7 @@ class question_usage_by_activity {
         while ($record->qubaid != $qubaid) {
             $records->next();
             if (!$records->valid()) {
-                throw new coding_exception("Question usage $qubaid not found in the database.");
+                throw new coding_exception("Question usage {$qubaid} not found in the database.");
             }
             $record = $records->current();
         }
@@ -867,8 +969,7 @@ class question_usage_by_activity {
 
 
 /**
- * A class abstracting access to the
- * {@link question_usage_by_activity::$questionattempts} array.
+ * A class abstracting access to the {@link question_usage_by_activity::$questionattempts} array.
  *
  * This class snapshots the list of {@link question_attempts} to iterate over
  * when it is created. If a question is added to the usage mid-iteration, it
@@ -881,15 +982,18 @@ class question_usage_by_activity {
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 class question_attempt_iterator implements Iterator, ArrayAccess {
+
     /** @var question_usage_by_activity that we are iterating over. */
     protected $quba;
-    /** @var array of question numbers. */
+
+    /** @var array of slot numbers. */
     protected $slots;
 
     /**
      * To create an instance of this class, use
      * {@link question_usage_by_activity::get_attempt_iterator()}.
-     * @param $quba the usage to iterate over.
+     *
+     * @param question_usage_by_activity $quba the usage to iterate over.
      */
     public function __construct(question_usage_by_activity $quba) {
         $this->quba = $quba;
@@ -897,37 +1001,83 @@ class question_attempt_iterator implements Iterator, ArrayAccess {
         $this->rewind();
     }
 
-    /** @return question_attempt_step */
+    /**
+     * Standard part of the Iterator interface.
+     *
+     * @return question_attempt
+     */
     public function current() {
         return $this->offsetGet(current($this->slots));
     }
-    /** @return int */
+
+    /**
+     * Standard part of the Iterator interface.
+     *
+     * @return int
+     */
     public function key() {
         return current($this->slots);
     }
+
+    /**
+     * Standard part of the Iterator interface.
+     */
     public function next() {
         next($this->slots);
     }
+
+    /**
+     * Standard part of the Iterator interface.
+     */
     public function rewind() {
         reset($this->slots);
     }
-    /** @return bool */
+
+    /**
+     * Standard part of the Iterator interface.
+     *
+     * @return bool
+     */
     public function valid() {
         return current($this->slots) !== false;
     }
 
-    /** @return bool */
+    /**
+     * Standard part of the ArrayAccess interface.
+     *
+     * @param int $slot
+     * @return bool
+     */
     public function offsetExists($slot) {
         return in_array($slot, $this->slots);
     }
-    /** @return question_attempt_step */
+
+    /**
+     * Standard part of the ArrayAccess interface.
+     *
+     * @param int $slot
+     * @return question_attempt
+     */
     public function offsetGet($slot) {
         return $this->quba->get_question_attempt($slot);
     }
+
+    /**
+     * Standard part of the ArrayAccess interface.
+     *
+     * @param int $slot
+     * @param question_attempt $value
+     */
     public function offsetSet($slot, $value) {
         throw new coding_exception('You are only allowed read-only access to ' .
                 'question_attempt::states through a question_attempt_step_iterator. Cannot set.');
     }
+
+    /**
+     * Standard part of the ArrayAccess interface.
+     *
+     * @param int $slot
+     */
     public function offsetUnset($slot) {
         throw new coding_exception('You are only allowed read-only access to ' .
                 'question_attempt::states through a question_attempt_step_iterator. Cannot unset.');
@@ -949,6 +1099,12 @@ interface question_usage_observer {
     /** Called when a field of the question_usage_by_activity is changed. */
     public function notify_modified();
 
+    /**
+     * Called when a new question attempt is added to this usage.
+     * @param question_attempt $qa the newly added question attempt.
+     */
+    public function notify_attempt_added(question_attempt $qa);
+
     /**
      * Called when the fields of a question attempt in this usage are modified.
      * @param question_attempt $qa the newly added question attempt.
@@ -956,10 +1112,11 @@ interface question_usage_observer {
     public function notify_attempt_modified(question_attempt $qa);
 
     /**
-     * Called when a new question attempt is added to this usage.
-     * @param question_attempt $qa the newly added question attempt.
+     * Called when a question_attempt has been moved to a new slot.
+     * @param question_attempt $qa The question attempt that was moved.
+     * @param int $oldslot The previous slot number of that attempt.
      */
-    public function notify_attempt_added(question_attempt $qa);
+    public function notify_attempt_moved(question_attempt $qa, $oldslot);
 
     /**
      * Called when a new step is added to a question attempt in this usage.
@@ -984,6 +1141,19 @@ interface question_usage_observer {
      */
     public function notify_step_deleted(question_attempt_step $step, question_attempt $qa);
 
+    /**
+     * Called when a new metadata variable is set on a question attempt in this usage.
+     * @param question_attempt $qa the question attempt the metadata is being added to.
+     * @param int $name the name of the metadata variable added.
+     */
+    public function notify_metadata_added(question_attempt $qa, $name);
+
+    /**
+     * Called when a metadata variable on a question attempt in this usage is updated.
+     * @param question_attempt $qa the question attempt where the metadata is being modified.
+     * @param int $name the name of the metadata variable modified.
+     */
+    public function notify_metadata_modified(question_attempt $qa, $name);
 }
 
 
@@ -997,9 +1167,11 @@ interface question_usage_observer {
 class question_usage_null_observer implements question_usage_observer {
     public function notify_modified() {
     }
+    public function notify_attempt_added(question_attempt $qa) {
+    }
     public function notify_attempt_modified(question_attempt $qa) {
     }
-    public function notify_attempt_added(question_attempt $qa) {
+    public function notify_attempt_moved(question_attempt $qa, $oldslot) {
     }
     public function notify_step_added(question_attempt_step $step, question_attempt $qa, $seq) {
     }
@@ -1007,4 +1179,8 @@ class question_usage_null_observer implements question_usage_observer {
     }
     public function notify_step_deleted(question_attempt_step $step, question_attempt $qa) {
     }
+    public function notify_metadata_added(question_attempt $qa, $name) {
+    }
+    public function notify_metadata_modified(question_attempt $qa, $name) {
+    }
 }
